Package: qtl2convert
Version: 0.34
Date: 2026-06-09
Title: Convert Data among QTL Mapping Packages
Description: Functions to convert data structures among the 'qtl2', 'qtl', and 'DOQTL' packages for mapping quantitative trait loci (QTL).
Author: Karl W Broman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4914-6671>)
Maintainer: Karl W Broman <broman@wisc.edu>
Authors@R: person("Karl W", "Broman", role=c("aut", "cre"),
                  email="broman@wisc.edu", comment=c(ORCID = "0000-0002-4914-6671"))
Depends: R (>= 3.1.0)
Imports: Rcpp (>= 0.12.12), qtl (>= 1.40-8), qtl2 (>= 0.18), utils,
        stats
Suggests: testthat, devtools, roxygen2
License: GPL-3
URL: https://kbroman.org/qtl2/, https://github.com/rqtl/qtl2convert
BugReports: https://github.com/rqtl/qtl2convert/issues
LinkingTo: Rcpp
Encoding: UTF-8
ByteCompile: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-09 12:33:21 UTC; kbroman
Repository: CRAN
Date/Publication: 2026-06-09 13:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-09 16:14:58 UTC; unix
Archs: qtl2convert.so.dSYM

Package: qtl2convert
Version: 0.36
Date: 2026-07-17
Title: Convert Data among QTL Mapping Packages
Description: Functions to convert data structures among the 'qtl2', 'qtl', and 'DOQTL' packages for mapping quantitative trait loci (QTL).
Author: Karl W Broman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4914-6671>)
Maintainer: Karl W Broman <broman@wisc.edu>
Authors@R: person("Karl W", "Broman", role=c("aut", "cre"),
                  email="broman@wisc.edu", comment=c(ORCID = "0000-0002-4914-6671"))
Depends: R (>= 3.1.0)
Imports: Rcpp (>= 0.12.12), qtl (>= 1.40-8), qtl2 (>= 0.18), utils,
        stats
Suggests: testthat, devtools, roxygen2
License: GPL-3
URL: https://kbroman.org/qtl2/, https://github.com/rqtl/qtl2convert
BugReports: https://github.com/rqtl/qtl2convert/issues
LinkingTo: Rcpp
Encoding: UTF-8
ByteCompile: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-17 19:41:10 UTC; kbroman
Repository: CRAN
Date/Publication: 2026-07-17 20:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-07-17 22:18:54 UTC; unix
Archs: qtl2convert.so.dSYM

# Generated by roxygen2: do not edit by hand

export(cbind_smother)
export(count_unique_geno)
export(cross2_do_to_genail)
export(cross2_do_to_genail8)
export(cross2_ril_to_genril)
export(encode_geno)
export(find_consensus_geno)
export(find_unique_geno)
export(map_df_to_list)
export(map_list_to_df)
export(probs_doqtl_to_qtl2)
export(probs_qtl2_to_array)
export(probs_qtl2_to_doqtl)
export(probs_qtl_to_qtl2)
export(scan_qtl2_to_qtl)
export(scan_qtl_to_qtl2)
export(write2csv)
importFrom(Rcpp,sourceCpp)
importFrom(qtl,getgenonames)
importFrom(qtl,getid)
importFrom(qtl,getsex)
importFrom(qtl,nind)
importFrom(qtl2,batch_vec)
importFrom(qtl2,cbind_expand)
importFrom(stats,setNames)
importFrom(utils,write.table)
useDynLib(qtl2convert, .registration=TRUE)

### R/qtl2convert

[![R-CMD-check](https://github.com/rqtl/qtl2convert/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rqtl/qtl2convert/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/qtl2convert)](https://cran.r-project.org/package=qtl2convert)
[![r-universe badge](https://rqtl.r-universe.dev/qtl2convert/badges/version)](https://rqtl.r-universe.dev/qtl2convert)
[![zenodo DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3237774.svg)](https://doi.org/10.5281/zenodo.3237774)

[R/qtl2](https://kbroman.org/qtl2/) (aka qtl2) is a reimplementation of
the QTL analysis software [R/qtl](https://rqtl.org), to better handle
high-dimensional data and complex cross designs.

The [qtl2convert](https://github.com/rqtl/qtl2convert) package is
for converting data among the R/qtl2,
[DOQTL](https://www.bioconductor.org/packages/3.9/bioc/html/DOQTL.html),
and [R/qtl](https://rqtl.org) formats.

---

### Installation

Install the qtl2convert package from [CRAN](https://cran.r-project.org):

```r
install.packages("qtl2convert")
```

Alternatively, install it from [R
universe](https://rqtl.r-universe.dev):

```r
install.packages("qtl2convert", repos=c("https://rqtl.r-universe.dev",
                                        "https://cloud.r-project.org"))
```

Or use [remotes](https://remotes.r-lib.org) to install it from its GitHub source:

```r
install.packages("remotes")
remotes::install_github("rqtl/qtl2convert")
```

---

### License

Licensed under [GPL-3](https://www.r-project.org/Licenses/GPL-3).

#' Combine matrices by columns, replacing matching ones and adding unique ones
#'
#' This is like [base::cbind()] but if a column in the second matrix
#' has the same name as a column in the first matrix, the column in
#' the first matrix is deleted and that in the second matrix is used
#' in its place.
#'
#' @param mat1 A matrix
#' @param mat2 Another matrix, with the same number of rows as `mat`.
#'
#' @return The two matrices combined by columns, but columns in the
#' first matrix that also appear in the second matrix are deleted
#' and replaced by those in the second matrix. Uses the row names
#' to align the rows in the two matrices, and to expand them as needed.
#'
#' @importFrom qtl2 cbind_expand
#' @export
#'
#' @examples
#' df1 <- data.frame(x=c(1,2,3,NA,4), y=c(5,8,9,10,11), row.names=c("A", "B", "C", "D", "E"))
#' df2 <- data.frame(z=c(7,8,0,9,10), y=c(6,NA,NA,9,10), row.names=c("A", "B", "F", "C", "D"))
#' df1n2 <- cbind_smother(df1, df2)
cbind_smother <-
  function(mat1, mat2)
{
    cn1 <- colnames(mat1)
    cn2 <- colnames(mat2)
    if(is.null(cn1) || is.null(cn2)) {
        stop("Need colnames for both mat1 and mat2")
    }

    m_col <- (cn2 %in% cn1)

    if(any(m_col)) {
        rn1 <- rownames(mat1)
        rn2 <- rownames(mat2)
        if(is.null(rn1) || is.null(rn2)) {
            stop("Need rownames for both mat1 and mat2")
        }
        m_row <- match(rn1[rn1 %in% rn2], rn2)

        mat1 <- qtl2::cbind_expand(mat1, mat2[,!m_col,drop=FALSE])
        mat1[rn2, m_col] <- mat2[rn2, m_col]
    }
    else {
        mat1 <- qtl2::cbind_expand(mat1, mat2)
    }

    mat1
}

context("cbind_smother")

test_that("cbind_smother works", {

    df1 <- data.frame(x=c(1,2,3,NA,4), y=c(5,8,9,10,11), row.names=c("A", "B", "C", "D", "E"))
    df2 <- data.frame(z=c(7,8,0,9,10), y=c(6,NA,NA,9,10), row.names=c("A", "B", "F", "C", "D"))

    df1n2 <- data.frame(x=c(1,2,3,NA,4,NA), y=c(6,NA,9,10,11,NA), z=c(7,8,9,10,NA,0),
                        row.names=c("A", "B", "C", "D", "E","F"))
    df2n1 <- data.frame(z=c(7,8,0,9,10,NA), y=c(5,8,NA,9,10,11), x=c(1,2,NA,3,NA,4),
                        row.names=c("A", "B", "F", "C", "D", "E"))

    expect_equal(cbind_smother(df1, df2), df1n2)
    expect_equal(cbind_smother(df2, df1), df2n1)

    expect_equal(cbind_smother(df1, df1), df1)
    expect_equal(cbind_smother(df2, df2), df2)

    expect_equal(cbind_smother(df1, df1[-3,]), df1)
    expect_equal(cbind_smother(df2[-2,], df2), df2[c(1,3,4,5,2),])

})

# scan_qtl2_to_qtl
#
#' Convert scan1 results to the scanone format
#'
#' Convert the results of [qtl2::scan1()]
#' to the form used by the R/qtl function
#' [qtl::scanone()].
#'
#' @param scan1_output Matrix of LOD scores, as calculated by
#' [qtl2::scan1()].
#' @param map Map of markers/pseudomarkers (as a list of vectors).
#'
#' @return A data frame with class `"scanone"`, containing
#' chromosome and position columns followed by the LOD scores in
#' `scan1_output`.
#'
#' @examples
#' library(qtl2)
#' iron <- read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
#' map <- insert_pseudomarkers(iron$gmap, step=1)
#' probs <- calc_genoprob(iron, map, error_prob=0.002)
#' pheno <- iron$pheno
#' covar <- match(iron$covar$sex, c("f", "m")) # make numeric
#' names(covar) <- rownames(iron$covar)
#' Xcovar <- get_x_covar(iron)
#' out <- scan1(probs, pheno, addcovar=covar, Xcovar=Xcovar)
#'
#' out_rev <- scan_qtl2_to_qtl(out, map)
#'
#' @seealso [scan_qtl2_to_qtl()]
#'
#' @export
scan_qtl2_to_qtl <-
    function(scan1_output, map)
{
    n <- vapply(map, length, 1)
    if(nrow(scan1_output) != sum(n))
        stop("nrow(scan1_output) [", nrow(scan1_output), "] != number of positions in map [",
             sum(n), "]")

    out <- data.frame(chr=factor(rep(names(map), n), levels=names(map)),
                      pos=unlist(map),
                      as.data.frame(unclass(scan1_output)))

    rownames(out) <- rownames(scan1_output)

    class(out) <- c("scanone", "data.frame")
    out
}

context("convert qtl2::scan1 to qtl::scanone output")

test_that("scan_qtl2_to_qtl works", {

    library(qtl)
    data(hyper)
    hyper <- calc.genoprob(hyper[c(6,8),], step=2.5, err=0.01)
    out_scanone <- scanone(hyper, method="hk")

    # pull out pseudomarker map and adjust names
    # (want to use the exact same map)
    pmap <- lapply(hyper$geno, function(a) {
        m <- attr(a$prob, "map")
        m[grep("^loc", names(m))] })
    for(i in seq(along=pmap))
        names(pmap[[i]]) <- paste0("c", names(pmap)[i], ".",
                                   names(pmap[[i]]))

    library(qtl2)
    hyper2 <- convert2cross2(hyper)
    map <- insert_pseudomarkers(pull.map(hyper), pseudomarker_map=pmap)

    pr <- calc_genoprob(hyper2, map, err=0.01)
    out_scan1 <- scan1(pr, hyper$pheno[,1,drop=FALSE])

    # adjustments to scanone output
    colnames(out_scanone)[3] <- "bp"
    for(atname in c("method", "type", "model"))
        attr(out_scanone, atname) <- NULL

    out_conv <- scan_qtl2_to_qtl(out_scan1, map)
    expect_equal(out_conv, out_scanone)

})

# used to skip tests when not on *my* computer in interactive mode
# needed for testing mult-core code, which isn't allowed within R CMD check
isnt_karl <-
    function()
{
    !(interactive() &&
      identical(Sys.getenv("KARL_LOCAL"), "true"))
}

context("find_unique_geno")

test_that("find_unique_geno works", {
    g <- rbind(c(NA,  "A",  "A",  "A", "T"),
               c(NA,  NA,   NA,   "A", "A"),
               c("A", "A",  "T",  "G", "G"),
               rep(NA, 5),
               c("N", "H",  "",   "A", "A"),
               c("N", "H",  "",   "A", "T"),
               c("C", "C",  "G",  "G", NA))
    expected <- rbind(c("A","T"),
                      c(NA, NA),
                      c(NA, NA),
                      c(NA, NA),
                      c(NA, NA),
                      c("A", "T"),
                      c("C", "G"))
    expect_equal(find_unique_geno(g), expected)

    rownames(g) <- rownames(expected) <- paste0("mar", 1:7)
    expect_equal(find_unique_geno(g), expected)

})

test_that("find_unique_geno works multi-core", {
    if(isnt_karl()) skip("this test only run locally")

    g <- rbind(c(NA,  "A",  "A",  "A", "T"),
               c(NA,  NA,   NA,   "A", "A"),
               c("A", "A",  "T",  "G", "G"),
               rep(NA, 5),
               c("N", "H",  "",   "A", "A"),
               c("N", "H",  "",   "A", "T"),
               c("C", "C",  "G",  "G", NA))
    expected <- rbind(c("A","T"),
                      c(NA, NA),
                      c(NA, NA),
                      c(NA, NA),
                      c(NA, NA),
                      c("A", "T"),
                      c("C", "G"))
    expect_equal(find_unique_geno(g), expected)
    expect_equal(find_unique_geno(g, cores=2), expected)

    rownames(g) <- rownames(expected) <- paste0("mar", 1:7)
    expect_equal(find_unique_geno(g), expected)
    expect_equal(find_unique_geno(g, cores=2), expected)
})

context("find_consensus_geno")

test_that("find_consensus_geno works", {
    g <- rbind(c("NA", "N", "A", "A", "T", "G", NA, "H"),
               c("C",  "C", "G", "G", "A", NA,  NA, NA),
               rep(NA, 8),
               c("C", "C", "G", "G", "G", "C", "G", "G"))
    expected <- c("A", NA, NA, "G")
    expect_equal(find_consensus_geno(g), expected)

    rownames(g) <- names(expected) <- paste0("mar", 1:4)
    expect_equal(find_consensus_geno(g), expected)

})

test_that("find_consensus_geno works multi-core", {
    if(isnt_karl()) skip("this test only run locally")

    g <- rbind(c("NA", "N", "A", "A", "T", "G", NA, "H"),
               c("C",  "C", "G", "G", "A", NA,  NA, NA),
               rep(NA, 8),
               c("C", "C", "G", "G", "G", "C", "G", "G"))
    expected <- c("A", NA, NA, "G")
    expect_equal(find_consensus_geno(g), expected)
    expect_equal(find_consensus_geno(g, cores=2), expected)

    rownames(g) <- names(expected) <- paste0("mar", 1:4)
    expect_equal(find_consensus_geno(g), expected)
    expect_equal(find_consensus_geno(g, cores=2), expected)

})

context("encode_geno")

test_that("encode_geno works", {

    geno <- rbind(c("C", "G", "C",  "GG", "CG"),
                  c("A", "A", "AT", "TA", "TT"),
                  c("T", "G", NA,   "GT", "TT"))
    codes <- rbind(c("C", "G"),
                   c("A", "T"),
                   c("T", "G"))

    expected <- rbind(c("A", "B", "A", "B", "H"),
                      c("A", "A", "H", "H", "B"),
                      c("A", "B", "-", "H", "A"))

    expect_equal(encode_geno(geno, codes), expected)

    expected2 <- gsub("Z", "A",
                      gsub("A", "B",
                           gsub("B", "Z", expected)))

    expect_equal(encode_geno(geno, cbind(codes[,2], codes[,1])), expected2)

    # errors: multiple characters, NAs
    expect_error(encode_geno(geno, cbind(paste0(codes[,1], codes[,1]), codes[,2])))
    expect_error(encode_geno(geno, cbind(codes[,1], paste0(codes[,2], codes[,2]))))
    codes[1,2] <- NA
    expect_error(encode_geno(geno, codes))

})


test_that("encode_geno works multi-core", {

    if(isnt_karl()) skip("this test only run locally")

    geno <- rbind(c("C", "G", "C",  "GG", "CG"),
                  c("A", "A", "AT", "TA", "TT"),
                  c("T", "G", NA,   "GT", "TT"))
    codes <- rbind(c("C", "G"),
                   c("A", "T"),
                   c("T", "G"))

    expected <- rbind(c("A", "B", "A", "B", "H"),
                      c("A", "A", "H", "H", "B"),
                      c("A", "B", "-", "H", "A"))

    expect_equal(encode_geno(geno, codes), expected)
    expect_equal(encode_geno(geno, codes, cores=2), expected)

    expected2 <- gsub("Z", "A",
                      gsub("A", "B",
                           gsub("B", "Z", expected)))

    expect_equal(encode_geno(geno, cbind(codes[,2], codes[,1])), expected2)
    expect_equal(encode_geno(geno, cbind(codes[,2], codes[,1]), cores=2), expected2)

})

context("cross2_do_to_genail")

test_that("cross2_do_to_genail works", {
    if(isnt_karl()) skip("this test only run locally")

    file <- paste0("https://raw.githubusercontent.com/rqtl/",
                   "qtl2data/master/DOex/DOex.zip")
    DOex <- read_cross2(file)

    DOex_genail <- cross2_do_to_genail(DOex)

    DOex$crosstype <- "genail8"
    DOex$cross_info <- cbind(DOex$cross_info + 5L,
                             A=1L, B=1L, C=1L, D=1L,
                             E=1L, F=1L, G=1L, H=1L)

    expect_equal(DOex_genail, DOex)

})

context("Convert genoprobs from DOQTL to qtl2")

test_that("probs_doqtl_to_qtl2 works", {

    n_ind <- 20
    n_gen <- 8
    n_pos <- 10
    d <- c(n_ind, n_gen, n_pos)
    p <- array(runif(prod(d)), dim=d)
    dimnames(p) <- list(paste0("ind", 1:n_ind),
                        LETTERS[1:8],
                        paste0("marker", 1:n_pos))


    map <- data.frame(marker=paste0("marker", 1:n_pos),
                      chr=rep(c("1", "2"), each=n_pos/2),
                      Mbp=c(sort(runif(5, 0, 100)), sort(runif(5, 0, 50))),
                      stringsAsFactors=FALSE)

    expect_error( probs_doqtl_to_qtl2(p, map) ) # no pos column
    result <- probs_doqtl_to_qtl2(p, map, pos_column="Mbp")

    expected <- list("1"=p[,,1:5], "2"=p[,,6:10])
    attr(expected, "is_x_chr") <- c("1"=FALSE,"2"=FALSE)
    attr(expected, "crosstype") <- "do"
    attr(expected, "alleles") <- LETTERS[1:8]
    attr(expected, "alleleprobs") <- TRUE
    class(expected) <- c("calc_genoprob", "list")

    expect_equal( result, expected )

})


test_that("probs convert between DOQTL and R/qtl2", {
    if(isnt_karl()) skip("this test only run locally")

    file <- paste0("https://raw.githubusercontent.com/rqtl/",
                   "qtl2data/master/DOex/DOex.zip")
    DOex <- read_cross2(file)

    # subset to chr 2
    DOex <- DOex[,"2"]

    # calculate genotype probabilities and convert to allele probabilities
    pr <- calc_genoprob(DOex, error_prob=0.002)

    map_tab <- map_list_to_df(DOex$gmap, pos_column="cM")

    # convert to DOQTL format
    pr_doqtl <- probs_qtl2_to_doqtl(pr)

    # convert back
    pr_rqtl2 <- probs_doqtl_to_qtl2(pr_doqtl, map_tab)

    expect_equal(pr_rqtl2, pr)

    # convert back yet again
    pr_doqtl_b <- probs_qtl2_to_doqtl(pr_rqtl2)

    expect_equal(pr_doqtl_b, pr_doqtl)

})

context("count_unique_geno")

test_that("count_unique_geno works", {

    g <- rbind(c(NA,  "A",  "A",  "A", "T"),
               c(NA,  NA,   NA,   "A", "A"),
               c("A", "A",  "T",  "G", "G"),
               rep(NA, 5),
               c("N", "H",  "",   "A", "A"),
               c("N", "H",  "",   "A", "T"),
               c("C", "C",  "G",  "G", NA))
    expected <- c(2,1,3,0,1,2,2)
    expect_equal(count_unique_geno(g), expected)

    rownames(g) <- names(expected) <- paste0("mar", 1:7)
    expect_equal(count_unique_geno(g), expected)

})


test_that("count_unique_geno works multi-core", {
    if(isnt_karl()) skip("this test only run locally")

    g <- rbind(c(NA,  "A",  "A",  "A", "T"),
               c(NA,  NA,   NA,   "A", "A"),
               c("A", "A",  "T",  "G", "G"),
               rep(NA, 5),
               c("N", "H",  "",   "A", "A"),
               c("N", "H",  "",   "A", "T"),
               c("C", "C",  "G",  "G", NA))
    expected <- c(2,1,3,0,1,2,2)
    expect_equal(count_unique_geno(g), expected)
    expect_equal(count_unique_geno(g, cores=2), expected)

    rownames(g) <- names(expected) <- paste0("mar", 1:7)
    expect_equal(count_unique_geno(g), expected)
    expect_equal(count_unique_geno(g, cores=2), expected)

})

# cross2_do_to_genail8()
#' Convert cross2 object from do to genail8
#'
#' Convert a cross2 object from cross type `"do"` to cross type `"genail8"`.
#'
#' @param cross Object of class `"cross2"`, as produced by
#' [qtl2::read_cross2()].
#'
#' @return The input object `cross` with cross type changed to
#' class `"genail8"` and the cross information revised to match.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' file <- paste0("https://raw.githubusercontent.com/rqtl/",
#'                "qtl2data/master/DOex/DOex.zip")
#' DOex <- read_cross2(file)
#'
#' DOex_genail <- cross2_do_to_genail8(DOex)
#' }

cross2_do_to_genail8 <-
    function(cross)
{

    # check that it's cross2
    if(!inherits(cross, "cross2")) stop('Input cross must have class "cross2"')

    # check that it's DO
    if(!cross$crosstype == "do") stop('input cross is not of type "do"')

    # crosstype -> genail8
    cross$crosstype <- "genail8"

    # revise cross_info ->
    cross$cross_info <- cbind(cross$cross_info+5L,
                                      A=1L, B=1L, C=1L, D=1L, E=1L, F=1L, G=1L, H=1L)

    cross
}


#' @rdname cross2_do_to_genail8
#' @export
cross2_do_to_genail <- cross2_do_to_genail8

#' Convert R/qtl genotype probabilities to R/qtl2 format
#'
#' Convert R/qtl genotype probabilities to R/qtl2 format
#'
#' @param cross An R/qtl `"cross"` object (see
#'     [qtl::read.cross()] for details.) Must contain
#'     genotype probabilities as calculated by
#'     [qtl::calc.genoprob()].
#'
#' @return A list with two components:
#' * `"probs"` - the genotype probabilities in the form produced by [qtl2::calc_genoprob()]
#' * `"map"` - Map of marker/pseudomarker positions (a list of vectors of positions)
#'
#' @examples
#' library(qtl)
#' data(hyper)
#' hyper <- calc.genoprob(hyper, step=1, error.prob=0.002)
#' result <- probs_qtl_to_qtl2(hyper)
#' pr <- result$probs
#' map <- result$map
#'
#' @importFrom stats setNames
#' @importFrom qtl getsex nind getgenonames getid
#' @export
probs_qtl_to_qtl2 <-
    function(cross)
{
    if(!inherits(cross, "cross"))
        stop("Input must be an R/qtl cross object")
    if(!("prob" %in% names(cross$geno[[1]])))
        stop("Input doesn't contain genotype probabilities from calc.genoprob")

    # treat bcsft as "f2" but return "bcsft" as the attribute
    crosstype <- qtl1_crosstype(cross)
    crosstype2 <- crosstype
    if(crosstype=="bcsft") crosstype2 <- "f2"

    chr <- names(cross$geno)

    # chromosome types -> is_x_chr
    chrtype <- sapply(cross$geno, qtl1_chrtype)
    chrtype[chrtype != "A" & chrtype != "X"] <- "A"
    is_x_chr <- stats::setNames(chrtype=="X", chr)

    # grab probabilities
    pr <- lapply(cross$geno, function(a) a$prob)

    # fix X chr if necessary
    if(any(is_x_chr) && (crosstype2=="bc" || crosstype2=="f2")) {
        sexpgm <- qtl::getsex(cross)
        for(j in seq_along(cross$geno)[is_x_chr])
            pr[[j]] <- revise_x_probs(crosstype2, sexpgm, prob=pr[[j]],
                                      attr(cross, "alleles"))
    }

    # reorder dimensions of the probabilities (ind,pos,genotype) -> (ind,genotype,pos)
    pr <- lapply(pr, function(a) aperm(a, c(1,3,2)))

    # grab map
    map <- lapply(cross$geno, function(a) attr(a$prob, "map"))

    # add chromosome names + individual IDs
    names(pr) <- names(map) <- chr
    ids <- qtl::getid(cross)
    if(is.null(ids)) ids <- 1:qtl::nind(cross)
    for(i in seq_along(pr)) rownames(pr[[i]]) <- ids

    attr(pr, "crosstype") <- crosstype
    attr(pr, "is_x_chr") <- is_x_chr
    attr(pr, "alleles") <- attr(cross, "alleles")
    attr(pr, "alleleprobs") <- FALSE
    class(pr) <- c("calc_genoprob", "list")

    list(probs=pr, map=map)
}


# convert the x chromosome probabilities for BC or intercross
revise_x_probs <-
    function(crosstype, sexpgm, prob, alleles)
{
    if(is.null(alleles)) alleles <- c("A", "B")
    gnames <- X_geno_names(crosstype, alleles)

    result <- array(0, dim=c(dim(prob)[1:2], length(gnames)))
    dimnames(result) <- list(rownames(prob), colnames(prob), gnames)

    if(crosstype=="bc") {
        sex <- sexpgm$sex
        if(is.null(sex)) stop("sex is missing")
        if(any(sex==0)) # females
            result[sex==0,,1:2] <- prob[sex==0,,]
        if(any(sex==1)) # males
            result[sex==1,,3:4] <- prob[sex==1,,]
    }
    else { # "f2"
        sex <- sexpgm$sex
        pgm <- sexpgm$pgm
        if(is.null(sex)) stop("sex is missing")
        if(is.null(pgm)) pgm <- rep(0, nrow(prob))
        femforw <- (sex==0 & pgm==0)
        femback <- (sex==0 & pgm==1)
        mal <- (sex==1)

        if(any(femforw))
            result[femforw,,1:2] <- prob[femforw,,]
        if(any(femback))
            result[femback,,3:4] <- prob[femback,,2:1]
        if(any(mal))
            result[mal,,5:6] <- prob[mal,,]
    }

    result
}

X_geno_names <-
    function(crosstype, alleles)
{
    # get X chr genotype names, forcing all columns (which is what R/qtl2 does)
    if(crosstype=="bc") {
        gn <- qtl::getgenonames(crosstype, "X", "full", list(sex=c(0,1), pgm=NULL),
                                list(alleles=alleles))
    } else {

        gn <- qtl::getgenonames(crosstype, "X", "full", list(sex=c(0,0,1), pgm=c(0,1,0)),
                                list(alleles=alleles))

        fr <- substr(gn, 3, 4)
        # things like ABf -> AB
        if(any(fr == "f"))
            gn[fr=="f"] <- substr(gn[fr=="f"], 1, 2)
        # things like ABr -> BA
        if(any(fr == "r"))
            gn[fr=="r"] <- vapply(strsplit(substr(gn[fr=="r"], 1, 2), ""),
                                  function(a) paste(rev(a), collapse=""), "")
    }
    gn
}

# R/qtl1 cross type
qtl1_crosstype <-
    function(cross)
    {
        type <- class(cross)
        type <- type[type != "cross" & type != "list"]
        if(length(type) > 1) {
            warning("cross has multiple classes")
        }
        type[1]
    }

# R/qtl1 chromosome type
qtl1_chrtype <-
    function(object)
    {
        if(inherits(object, "X")) return("X")
        "A"
    }

context("probs_qtl_to_qtl2")

test_that("probs_qtl_to_qtl2 works for backcross", {

    library(qtl)
    library(qtl2)

    data(hyper)
    hyper <- hyper[c(1,6,"X"),]
    hyper <- calc.genoprob(hyper, step=1, error.prob=0.002)

    hyper2 <- qtl2::convert2cross2(hyper)
    m <- lapply(hyper$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(hyper2, m, error_prob=0.002)

    result <- probs_qtl_to_qtl2(hyper)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr, tolerance=1e-5)

})


test_that("probs_qtl_to_qtl2 works for intercross", {

    set.seed(92493498)

    library(qtl)
    library(qtl2)

    data(listeria)
    listeria <- listeria[c(1,8,"X"),]
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)

    listeria2 <- qtl2::convert2cross2(listeria)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)

    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

    # all males
    listeria$pheno$sex <- "male"
    listeria2 <- qtl2::convert2cross2(listeria)
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)
    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

    # all females, backward direction
    listeria$pheno$sex <- "female"
    listeria$pheno$pgm <- 1
    listeria2 <- qtl2::convert2cross2(listeria)
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)
    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

    # all females, both directions
    listeria$pheno$pgm <- sample(0:1, nind(listeria), replace=TRUE)
    listeria2 <- qtl2::convert2cross2(listeria)
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)
    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

    # males and females, forward
    listeria$pheno$sex <- sample(c("female", "male"), nind(listeria), replace=TRUE)
    listeria$pheno$pgm <- 0
    listeria2 <- qtl2::convert2cross2(listeria)
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)
    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

    # males and females, backward
    listeria$pheno$sex <- sample(c("female", "male"), nind(listeria), replace=TRUE)
    listeria$pheno$pgm <- 1
    listeria2 <- qtl2::convert2cross2(listeria)
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)
    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

    # both sex, both directions
    listeria$pheno$sex <- sample(c("female", "male"), nind(listeria), replace=TRUE)
    listeria$pheno$pgm <- sample(0:1, nind(listeria), replace=TRUE)
    listeria2 <- qtl2::convert2cross2(listeria)
    listeria <- calc.genoprob(listeria, step=1, error.prob=0.002)
    m <- lapply(listeria$geno, function(a) attr(a$prob, "map"))
    pr <- calc_genoprob(listeria2, m, error_prob=0.002)
    result <- probs_qtl_to_qtl2(listeria)
    expect_equal(result$map, m)
    expect_equal(result$probs, pr)

})

#' Convert R/qtl2 genotype probabilities to a 3d array
#'
#' Convert R/qtl2 genotype probabilities to a 3d array
#'
#' @param probs A `"calc_genoprob"` object (a list of 3d arrays of
#' genotype probabilities), as calculated by [qtl2::calc_genoprob()].
#'
#' @return A single three-dimensional array, with just the autosomal
#' genotype probabilities.
#'
#' @details
#' We convert just the autosomal genotype probabilities, because they
#' should all have the same number of genotypes (columns). The main
#' application of this is for identifying possible sample mix-ups
#' among batches of genotype probabilities (e.g., using the
#' [R/lineup2](https://github.com/kbroman/lineup2) package), and for
#' this the autosomal genotype probabilities should be sufficient.
#'
#' @importFrom stats setNames
#' @export
probs_qtl2_to_array <-
    function(probs)
{
    # which is the X chromosome?
    is_x_chr <- attr(probs, "is_x_chr")
    if(is.null(is_x_chr)) is_x_chr <- rep(FALSE, length(probs))

    # reduce to autosomes
    probs <- unclass(probs)[!is_x_chr]

    # dimensions
    d <- vapply(probs, dim, 1:3)

    if(length(unique(d[1,])) != 1)
        stop("Different chromosomes have different numbers of individuals")

    if(length(unique(d[2,])) != 1)
        stop("Different chromosomes have different numbers of genotypes")

    result <- array(dim=setNames(c(d[1,1], d[2,1], sum(d[3,])), NULL))
    dimnames(result) <- list(rownames(probs[[1]]),
                             colnames(probs[[1]]),
                             unlist(lapply(probs, function(a) dimnames(a)[[3]])))

    cur <- 0
    for(i in seq_along(probs)) {
        result[,,1:d[3,i]] <- probs[[i]]
        cur <- cur + d[3,i]
    }

    result
}

#' Convert DOQTL genotype probabilities to R/qtl2 format
#'
#' Convert DOQTL genotype probabilities to R/qtl2 format
#'
#' @param probs 3d array of genotype probabilities as calculated from DOQTL
#' (individuals x genotypes x positions)
#'
#' @param map Data frame with marker map
#'
#' @param is_female Optional logical vector indicating which
#' individuals are female. Names should contain the individual
#' identifiers, matching the row names in `probs`.
#'
#' @param chr_column Name of the column in `map` that contains the chromosome IDs.
#'
#' @param pos_column Name of the column in `map` that contains the marker positions.
#'
#' @param marker_column Name of the column in `map` that contains
#' the marker names. If NULL, use the row names.
#'
#' @return An object of the form produced by [qtl2::calc_genoprob()].
#'
#' @details
#' We assume that the X chromosome is labeled `"X"` (must be
#' upper-case) and that any other chromosomes are autosomes.
#' We assume that the genotypes are labeled using the 8 letters A-H.
#'
#' If the probabilities are for the full 36 states and the X
#' chromosome is included but `is_female` is not provided, we'll guess
#' which individuals are females based on their genotype
#' probabilities. (If the average, across loci, of the sum of the
#' heterozygote probabilities is small, we'll assume it's a female.)
#'
#' @export
probs_doqtl_to_qtl2 <-
    function(probs, map, is_female=NULL, chr_column="chr", pos_column="cM", marker_column="marker")
{
    if(is.null(marker_column)) {
        marker_column <- "qtl2tmp_marker"
        map[,marker_column] <- rownames(marker_column)
    }
    if(!(marker_column %in% colnames(map)))
        stop('Column "', marker_column, '" not found.')
    if(!(chr_column %in% colnames(map)))
        stop('Column "', chr_column, '" not found.')
    if(!(pos_column %in% colnames(map)))
        stop('Column "', pos_column, '" not found.')

    marker <- map[,marker_column]

    # subset map to the portion in probs
    map <- map[marker %in% dimnames(probs)[[3]],]
    marker <- map[,marker_column]

    # subset probs to the portion in map
    in_map <- dimnames(probs)[[3]] %in% marker
    if(sum(in_map)==0) {
        stop("No markers in common between probs and map")
    }
    if(any(!in_map)) {
        probs <- probs[,,in_map, drop=FALSE]
        warning("Omitting ", sum(!in_map), " markers from probs, not found in map")
    }

    # make sure they're aligned
    m <- match(marker, dimnames(probs)[[3]])
    probs <- probs[,,m, drop=FALSE]

    # number of genotypes
    n_geno <- ncol(probs)
    if(n_geno != 36 & n_geno != 8)
        stop("number of genotypes (number of columns of probs) should be 8 or 36")

    # align probabilities and is_female
    if(!is.null(is_female) && n_geno==36) { # if n_geno==8, ignore is_female
        ids1 <- names(is_female)
        ids2 <- rownames(probs)
        if(is.null(ids1) || is.null(ids2))
            stop("probs and is_female must both contain individual identifiers")
        both <- ids2[ids2 %in% ids1] # take order from probs
        if(length(both) == 0)
            stop("No individuals in common between probs and is_female")
        n_sex_only <- sum(!(ids1 %in% both))
        n_probs_only <- sum(!(ids2 %in% both))
        if(n_sex_only > 0) {
            warning("Omitting ", n_sex_only, " individuals absent from probs")
        }
        is_female <- is_female[both] # reorder no matter what
        if(n_probs_only > 0) {
            warning("Omitting ", n_probs_only, " individuals absent from is_female")
            probs <- probs[both,,,drop=FALSE]
        }
    }

    # reorder from (AA, AB, AC, AD, AE, ...) to (AA, AB, BB, AC, BC, CC, AD, ...)
    mat <- rbind(rep(1:8, each=8), rep(1:8, 8))
    old_geno_labels <- apply(mat[,mat[1,] <= mat[2,]], 2, function(a) paste(LETTERS[a], collapse=""))

    new_geno_labels <- c("AA", "AB", "BB", "AC", "BC", "CC", "AD", "BD", "CD", "DD", "AE", "BE",
                         "CE", "DE", "EE", "AF", "BF", "CF", "DF", "EF", "FF", "AG", "BG", "CG",
                         "DG", "EG", "FG", "GG", "AH", "BH", "CH", "DH", "EH", "FH", "GH", "HH")

    new_geno_order <- match(new_geno_labels, old_geno_labels)

    # split probs (and reorder if necessary)
    chr <- map[,chr_column]
    if(is.factor(chr)) {
        chr <- factor(chr) # reduce to observed levels
        uchr <- levels(chr)
    }
    else uchr <- unique(chr)

    newprobs <- setNames(vector("list", length(uchr)), uchr)
    for(i in uchr) {
        if(n_geno == 36) {
            newprobs[[i]] <- probs[,new_geno_order,chr==i,drop=FALSE]
        } else {
            newprobs[[i]] <- probs[,, chr==i,drop=FALSE]
        }

        if(i == "X" && n_geno==36) { # need to pull apart the males and females
            newprobs[[i]] <- array(0, dim=c(nrow(probs), ncol(probs)+8, sum(chr==i)))
            dimnames(newprobs[[i]]) <- list(rownames(probs),
                                            c(new_geno_labels, paste0(LETTERS[1:8], "Y")),
                                            dimnames(probs)[[3]][chr==i])
            hom <- paste0(LETTERS[1:8], LETTERS[1:8])

            if(is.null(is_female)) { # need to guess sex
                het_tolerance <- 1e-4

                het <- old_geno_labels[!(old_geno_labels %in% hom)]
                sum_het <- apply(probs[,het,chr==i,drop=FALSE], 1, sum)/dim(probs)[[3]]
                is_female <- setNames((sum_het > het_tolerance), rownames(probs))
            }
            if(any(is_female)) {
                newprobs[[i]][is_female, 1:36, ] <- probs[is_female, new_geno_order, chr==i, drop=FALSE]
            }
            if(any(!is_female)) {
                newprobs[[i]][!is_female, 37:44, ] <- probs[!is_female, hom, chr==i, drop=FALSE]
            }
        }
    }

    is_x_chr <- (uchr=="X")
    names(is_x_chr) <- uchr

    if(n_geno == 8) {
        alleleprobs <- TRUE
    } else {
        alleleprobs <- FALSE
    }

    attr(newprobs, "is_x_chr") <- is_x_chr
    attr(newprobs, "crosstype") <- "do"
    attr(newprobs, "alleles") <- LETTERS[1:8]
    attr(newprobs, "alleleprobs") <- alleleprobs

    class(newprobs) <- c("calc_genoprob", "list")
    newprobs
}

#' Convert R/qtl2 genotype probabilities to DOQTL format
#'
#' Convert R/qtl2 genotype probabilities to DOQTL format
#'
#' @param probs A `"calc_genoprob"` object (a list of 3d arrays of
#' genotype probabilities), as calculated by [qtl2::calc_genoprob()].
#'
#' @return A single three-dimensional array, for use with
#'     [DOQTL](https://github.com/dmgatti/DOQTL).
#'
#' @details
#' If the arrays in `probs` all have 8 columns, they're assumed to be
#' allele dosages and we paste them all together into one big array.
#'
#' Otherwise, it should be that the autosomes all have 36 columns the
#' X chromosome has 44. In this case, the male hemizygotes on the X
#' are placed where the female homozygotes are, and then we reorder
#' the genotypes into alphabetical order.
#'
#' @export
probs_qtl2_to_doqtl <-
    function(probs)
{
    is_x_chr <- attr(probs, "is_x_chr")
    n_geno <- sapply(probs, ncol)

    crosstype <- attr(probs, "crosstype")
    if(is.null(crosstype))
        warning('Expecting crosstype "do" but it is missing.')
    else if(crosstype!="do")
        warning('Expecting crosstype "do" but it is "', crosstype, '"')

    n_pos <- sapply(probs, function(a) dim(a)[3])

    if(all(n_geno==8)) { # allele dosages

        # big array to contain the results
        result <- array(dim=c(nrow(probs[[1]]), ncol(probs[[1]]), sum(n_pos)))
        dimnames(result) <- list(rownames(probs[[1]]), colnames(probs[[1]]),
                                 unlist(lapply(probs, function(a) dimnames(a)[[3]])))

        # copy over one chromosome at a time
        cur <- 0
        for(i in seq_along(probs)) {
            result[,,cur + 1:n_pos[i]] <- probs[[i]]
            cur <- cur + n_pos[i]
        }

        return(result)
    }
    else { # genotype probabilities
        if(!all(n_geno[is_x_chr] == 44) || !all(n_geno[!is_x_chr]==36))
            stop("There should be 44 genotypes on the X and 36 on the autosomes.")

        # get allele labels
        alleles <- attr(probs, "alleles")
        if(is.null(alleles)) alleles <- LETTERS[1:8]
        if(length(alleles) != 8)
            stop("length(alleles) != 8")

        # deal with X chromosome (paste male hemizygotes where the female homozygotes are)
        homozyg <- paste0(alleles, alleles)
        hemizyg <- paste0(alleles, "Y")
        for(chr in names(probs)[is_x_chr]) { # X chr
            probs[[chr]][,homozyg,] <- probs[[chr]][,homozyg,,drop=FALSE] + probs[[chr]][,hemizyg,,drop=FALSE]
            probs[[chr]] <- probs[[chr]][,1:36,,drop=FALSE]
        }

        # new order of genotype labels
        mat <- rbind(rep(1:8, each=8), rep(1:8, 8))
        geno_labels <- apply(mat[,mat[1,] <= mat[2,]], 2, function(a) paste(alleles[a], collapse=""))

        # big array to contain the results
        result <- array(dim=c(nrow(probs[[1]]), ncol(probs[[1]]), sum(n_pos)))
        dimnames(result) <- list(rownames(probs[[1]]), geno_labels,
                                 unlist(lapply(probs, function(a) dimnames(a)[[3]])))

        # copy over one chromosome at a time, also re-ordering the genotypes
        cur <- 0
        for(i in seq_along(probs)) {
            result[,,cur + 1:n_pos[i]] <- probs[[i]][,geno_labels,,drop=FALSE]
            cur <- cur + n_pos[i]
        }

        return(result)

    }

}

# encode_geno
#' Encode a matrix of genotypes using a set of allele codes
#'
#' Encode a matrix of genotypes using a set of allele codes.
#'
#' @param geno Character matrix of genotypes (rows as markers, columns as individuals)
#' @param allele_codes Two-column matrix of alleles (rows as markers)
#' @param output_codes Vector of length four, with missing, AA, AB, BB codes
#' @param cores Number of CPU cores to use, for parallel calculations.
#' (If `0`, use [parallel::detectCores()].)
#' Alternatively, this can be links to a set of cluster sockets, as
#' produced by [parallel::makeCluster()].
#'
#' @return Matrix of same dimensions as `geno`, but with values in `output_codes`.
#'
#' @seealso [find_consensus_geno()], [find_unique_geno()]
#'
#' @export
#' @importFrom qtl2 batch_vec
#'
#' @examples
#' geno <- rbind(c("C", "G", "C",  "GG", "CG"),
#'               c("A", "A", "AT", "TA", "TT"),
#'               c("T", "G", NA,   "GT", "TT"))
#' codes <- rbind(c("C", "G"), c("A", "T"), c("T", "G"))
#' geno_encoded <- encode_geno(geno, codes)
#'
encode_geno <-
function(geno, allele_codes, output_codes=c("-", "A", "H", "B"), cores=1)
{
    stopifnot(nrow(geno) == nrow(allele_codes))
    stopifnot(ncol(allele_codes) == 2)

    if(any(is.na(allele_codes))) stop("allele_codes can't have missing values")
    if(any(apply(allele_codes, 2, nchar) > 1)) stop("allele_codes should be single characters")

    if(!is.matrix(geno)) geno <- as.matrix(geno)
    if(!is.matrix(allele_codes)) allele_codes <- as.matrix(allele_codes)

    # if no rows, nothing to do
    if(nrow(geno)==0) return(geno)

    geno_codes <- cbind(allele_codes, # A and B
                        paste0(allele_codes[,1], allele_codes[,1]), # AA
                        paste0(allele_codes[,2], allele_codes[,2]), # BB
                        paste0(allele_codes[,1], allele_codes[,2]), # AB
                        paste0(allele_codes[,2], allele_codes[,1])) # BA

    cores <- setup_cluster(cores)
    by_batch_func <- function(batch) .encode_geno(geno[batch,,drop=FALSE], geno_codes[batch,,drop=FALSE],
                                                  output_codes[c(2,4,2,4,3,3)])
    batches <- batch_vec(1:nrow(geno), n_cores=n_cores(cores))

    result <- cluster_lapply(cores, batches, by_batch_func)
    result <- do.call("rbind", result)
    result[is.na(result)] <- output_codes[1]

    dimnames(result) <- dimnames(geno)
    result
}

# find_consensus_geno
#' Find the consensus genotype for each row of a genotype matrix
#'
#' For genotype data (markers x individuals) on a set of individuals
#' from a single inbred line, find the consensus genotype at each
#' marker.
#'
#' @param genotypes Matrix of genotypes (markers x individuals)
#' @param na.strings Genotypes to be considered as missing values.
#' @param cores Number of CPU cores to use, for parallel calculations.
#' (If `0`, use [parallel::detectCores()].)
#' Alternatively, this can be links to a set of cluster sockets, as
#' produced by [parallel::makeCluster()].
#'
#' @return Vector of consensus genotypes, one value per row of `genotypes`
#'
#' @seealso [find_unique_geno()], [encode_geno()]
#'
#' @export
#' @importFrom qtl2 batch_vec
#'
#' @examples
#' g <- rbind(c("NA", "N", "A", "A", "T", "G", NA, "H"),
#'            c("C",  "C", "G", "G", "A", NA,  NA, NA),
#'            rep(NA, 8),
#'            c("C", "C", "G", "G", "G", "C", "G", "G"))
#' consensus <- find_consensus_geno(g)
find_consensus_geno <-
function(genotypes, na.strings=c("N", "H", "NA", ""), cores=1)
{
    genotypes[is.na(genotypes) | (genotypes %in% na.strings)]  <- "NA"

    if(!is.matrix(genotypes)) genotypes <- as.matrix(genotypes)

    # if no rows, return NULL
    if(nrow(genotypes)==0) return(NULL)

    cores <- setup_cluster(cores)

    by_batch_func <- function(batch) .find_consensus_geno(genotypes[batch,,drop=FALSE])

    batches <- batch_vec(1:nrow(genotypes), n_cores=n_cores(cores))

    result <- unlist(cluster_lapply(cores, batches, by_batch_func))
    result[result=="NA"] <- NA
    names(result) <- rownames(genotypes)

    result
}

# find_unique_geno
#' Find the unique genotypes for each row of a genotype matrix
#'
#' For genotype data (markers x individuals) on a set of individuals,
#' find the unique genotypes for each marker, provided that there are exactly two.
#' (If more than two or fewer than two, return NAs.)
#'
#' @param genotypes Matrix of genotypes (markers x individuals)
#' @param na.strings Genotypes to be considered as missing values.
#' @param cores Number of CPU cores to use, for parallel calculations.
#' (If `0`, use [parallel::detectCores()].)
#' Alternatively, this can be links to a set of cluster sockets, as
#' produced by [parallel::makeCluster()].
#'
#' @return Matrix with two columns. Each row corresponds to a marker,
#' and has the two unique genotypes, or `NA`s (if >2 or <2 unique
#' genotypes).
#'
#' @seealso [count_unique_geno()], [encode_geno()]
#'
#' @export
#' @importFrom qtl2 batch_vec
#'
#' @examples
#' g <- rbind(c("NA", "A",  "A",  "A", "T"),
#'            c("NA", "NA", "NA", "A", "A"),
#'            c("A",  "A",  "T",  "G", "G"),
#'            c("C", "C",  "G",  "G", "NA"))
#' ug <- find_unique_geno(g)
find_unique_geno <-
function(genotypes, na.strings=c("N", "H", "NA", ""), cores=1)
{
    genotypes[is.na(genotypes) | (genotypes %in% na.strings)]  <- "NA"

    # force to be a matrix
    if(!is.matrix(genotypes)) genotypes <- as.matrix(genotypes)

    # if no rows, return NULL
    if(nrow(genotypes)==0) return(NULL)

    cores <- setup_cluster(cores)

    by_batch_func <- function(batch) .find_unique_geno(genotypes[batch,,drop=FALSE])

    batches <- batch_vec(1:nrow(genotypes), n_cores=n_cores(cores))

    result <- cluster_lapply(cores, batches, by_batch_func)
    result <- do.call("rbind", result)
    result[result=="NA"] <- NA
    rownames(result) <- rownames(genotypes)

    result
}

#' @keywords internal
#' @importFrom Rcpp sourceCpp
#' @useDynLib qtl2convert, .registration=TRUE
#'
#' @section Vignettes:
#' * [user guide](https://kbroman.org/qtl2/assets/vignettes/user_guide.html)
#' * [input file formats](https://kbroman.org/qtl2/assets/vignettes/input_files.html)
#' * [preparing DO mouse data for R/qtl2](https://kbroman.org/qtl2/pages/prep_do_data.html)
#' * [differences between R/qtl and R/qtl2](https://kbroman.org/qtl2/assets/vignettes/rqtl_diff.html)
#' * [developer guide](https://kbroman.org/qtl2/assets/vignettes/developer_guide.html)
"_PACKAGE"

# count_unique_geno
#' Count the unique genotypes for each row of a genotype matrix
#'
#' For genotype data (markers x individuals) on a set of individuals,
#' count the unique genotypes for each marker
#'
#' @param genotypes Matrix of genotypes (markers x individuals)
#' @param na.strings Genotypes to be considered as missing values.
#' @param cores Number of CPU cores to use, for parallel calculations.
#' (If `0`, use [parallel::detectCores()].)
#' Alternatively, this can be links to a set of cluster sockets, as
#' produced by [parallel::makeCluster()].
#'
#' @return Vector of counts of unique genotypes.
#'
#' @seealso [find_unique_geno()]
#'
#' @export
#' @importFrom qtl2 batch_vec
#'
#' @examples
#' g <- rbind(c("NA", "A",  "A",  "A", "T"),
#'            c("NA", "NA", "NA", "A", "A"),
#'            c("A",  "A",  "T",  "G", "G"),
#'            c("C", "C",  "G",  "G", "NA"))
#' counts <- count_unique_geno(g)
count_unique_geno <-
function(genotypes, na.strings=c("N", "H", "NA", ""), cores=1)
{
    genotypes[is.na(genotypes) | (genotypes %in% na.strings)]  <- "NA"

    # force to be a matrix
    if(!is.matrix(genotypes)) genotypes <- as.matrix(genotypes)

    # if no rows, return NULL
    if(nrow(genotypes)==0) return(NULL)

    cores <- setup_cluster(cores)

    by_batch_func <- function(batch) .count_unique_geno(genotypes[batch,,drop=FALSE])

    batches <- batch_vec(1:nrow(genotypes), n_cores=n_cores(cores))

    result <- unlist(cluster_lapply(cores, batches, by_batch_func))
    names(result) <- rownames(genotypes)

    result
}

# cross2_ril_to_genail()
#' Convert cross2 object from ril to genril
#'
#' Convert a cross2 object from cross type `"risibn"` to cross type `"genriln"`.
#'
#' @param cross Object of class `"cross2"`, as produced by
#' [qtl2::read_cross2()].
#'
#' @return The input object `cross` with cross type changed to
#' class `"genriln"` for some value of `n`,
#' and the cross information revised to match.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' file <- paste0("https://raw.githubusercontent.com/rqtl/",
#'                "qtl2data/master/CC/cc.zip")
#' cc <- read_cross2(file)
#'
#' cc_genril <- cross2_ril_to_genril(cc)
#' }
#'
#' \dontrun{
#' file <- paste0("https://raw.githubusercontent.com/rqtl/",
#'                "qtl2data/master/ArabMAGIC/arabmagic_tair9.zip")
#' arab <- read_cross2(file)
#'
#' arab_genril <- cross2_ril_to_genril(arab)
#' }

cross2_ril_to_genril <-
    function(cross)
{

    # check that it's cross2
    if(!inherits(cross, "cross2")) stop('Input cross must have class "cross2"')

    # check that it's ril
    crosstype <- cross$crosstype
    valid_types <- c("riself4", "risib4",
                     "riself8", "risib8",
                     "riself16", "magic19", "dh6")

    if(!(crosstype %in% valid_types)) {
        stop('input cross is not one of the allowed types')
    }

    n_str <- as.numeric(gsub("[A-Za-z]+", "", crosstype))
    if(is.na(n_str)) {
        stop("Can't determine the number of strains")
    }

    # crosstype -> genril
    cross$crosstype <- paste0("genril", n_str)

    # equivalent number of generations
    if(grepl("^risib", crosstype)) {
        n_gen <- ceiling((log2(n_str)+4) * 8/7)
    } else { # selfing or just generally
        n_gen <- ceiling((log2(n_str)+1) * 8/7)
    }

    # revise cross_info ->
    new_cross_info <- cbind(n_gen = as.integer(n_gen),
                            matrix(1L, ncol=n_str, nrow=nrow(cross$cross_info)))
    dimnames(new_cross_info) <- list(rownames(cross$cross_info), c("n_gen", LETTERS[seq_len(n_str)]))
    cross$cross_info <- new_cross_info

    cross
}

#' Marker map list to data frame
#'
#' Convert a marker map organized as a list to a data frame
#'
#' @param map_list List of vectors containing marker positions
#'
#' @param chr_column Name of the chromosome column in the output
#'
#' @param pos_column Name of the position column in the output
#'
#' @param marker_column Name of the marker column in the output.
#' If NULL, just put them as row names.
#'
#' @seealso [map_df_to_list()]
#'
#' @return A data frame with the marker positions.
#'
#' @examples
#' library(qtl2)
#' iron <- read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
#' iron_map <- map_list_to_df(iron$gmap)
#'
#' @export
map_list_to_df <-
    function(map_list, chr_column="chr", pos_column="pos", marker_column="marker")
{
    nmar <- vapply(map_list, length, 1) # no. markers per chromosome

    markers <- unlist(lapply(map_list, names))

    result <- data.frame(chr=rep(names(map_list), nmar),
                         pos=unlist(map_list),
                         marker=markers,
                         stringsAsFactors=FALSE)
    rownames(result) <- markers

    names(result)[1] <- chr_column
    names(result)[2] <- pos_column
    if(is.null(marker_column))
        result <- result[,-3,drop=FALSE]
    else
        names(result)[3] <- marker_column

    result
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.count_unique_geno <- function(g) {
    .Call(`_qtl2convert_count_unique_geno`, g)
}

.encode_geno <- function(g, old_values, new_values) {
    .Call(`_qtl2convert_encode_geno`, g, old_values, new_values)
}

.find_consensus_geno <- function(g) {
    .Call(`_qtl2convert_find_consensus_geno`, g)
}

.find_unique_geno <- function(g) {
    .Call(`_qtl2convert_find_unique_geno`, g)
}

#' Convert R/qtl scanone results to R/qtl2 scan1 format
#'
#' Convert the results of R/qtl1 [qtl::scanone()] to the form used by the R/qtl2 [qtl2::scan1()].
#'
#' @param scanone_output Data frame as output by the R/qtl1 function [qtl::scanone()].
#'
#' @return List with two objects: the LOD scores in [qtl2::scan1()]
#'     format, and the map (as a list of marker/pseudomarker
#'     positions).
#'
#' @examples
#' library(qtl)
#' data(hyper)
#' hyper <- calc.genoprob(hyper, step=1, error.prob=0.002)
#' out <- scanone(hyper)
#' out2 <- scan_qtl_to_qtl2(out)
#'
#' @seealso [scan_qtl_to_qtl2()]
#'
#' @export
scan_qtl_to_qtl2 <-
    function(scanone_output)
{
  scan1 <- as.matrix(scanone_output[,-(1:2),drop=FALSE])
  class(scan1) <- c("scan1", "matrix")

  map_df <- cbind(as.data.frame(scanone_output[,1:2]), marker=rownames(scanone_output))
  map_list <- map_df_to_list(map_df, pos_column="pos")

  list(scan1=scan1, map=map_list)
}

# write2csv
#' Write a data frame to a CSV file
#'
#' Write a data frame to a CSV file in a special form, with info about the number of rows and columns.
#'
#' @param df A data frame (or matrix)
#' @param filename File name to write
#' @param comment Comment to place on the first line
#' @param sep Field separator
#' @param comment.char Character to use to initiate the comment lines
#' @param row.names If NA or NULL (the default), row names are not
#' included in the output file. Otherwise, the row names are
#' included as the first column of the output, and this is taken
#' to be the name for that column.
#' @param overwrite If TRUE, overwrite file if it exists. If FALSE
#' (the default) and the file exists, stop with an error.
#'
#' @return None.
#'
#' @details
#' If the file already exists, the function will refuse to write over it.
#'
#' The file will include comments at the top, using `#` as a
#' comment character, including the number of rows (not including the
#' header) and the number of columns.
#'
#' By default, row names are not included. But with the option
#' `row.names` provided as a character string, they are added as an
#' initial column, with the value of this argument defining the name
#' for that column. If a column with that name already exists, the
#' function halts with an error.
#'
#' @importFrom utils write.table
#' @export
#'
#' @examples
#' nr <- 10
#' nc <- 5
#' x <- data.frame(id=paste0("ind", 1:nr),
#'                 matrix(rnorm(nr*nc), ncol=nc))
#' colnames(x)[1:nc + 1] <- paste0("col", 1:nc)
#'
#' testfile <- file.path(tempdir(), "tmpfile.csv")
#' write2csv(x, testfile, "A file created by write2csv")
#'
#' # Remove the file, to clean up temporary directory
#' unlink(testfile)
write2csv <-
    function(df, filename, comment="", sep=",", comment.char="#",
             row.names=NULL, overwrite=FALSE)
{
    if(!overwrite && file.exists(filename))
        stop(filename, " already exits. Remove it first (or use overwrite=TRUE).")

    if(!is.null(row.names) && !is.na(row.names)) {
        if(length(row.names) > 1) {
            row.names <- row.names[1]
            warning("row.names should have length 1")
        }
        if(!is.character(row.names)) {
            row.names <- as.character(row.names)
            warning("row.names should be a character string")
        }
        if(row.names %in% colnames(df)) {
            stop('There is already a column named "', row.names, '"')
        }
        if(is.null(rownames(df))) {
            rownames(df) <- seq_len(nrow(df))
        }

        df <- cbind(rownames(df), df)
        colnames(df)[1] <- row.names
    }

    if(comment != '')
        cat(comment.char, " ", comment, "\n", file=filename, sep="")

    cat(comment.char, " nrow ", nrow(df), "\n", file=filename, append=(comment!=''),
        sep="")

    cat(comment.char, " ncol ", ncol(df), "\n", file=filename, append=TRUE, sep="")

    cat(paste(colnames(df), collapse=sep), "\n", file=filename, append=TRUE, sep="")

    write.table(df, file=filename, append=TRUE,
                sep=sep, row.names=FALSE, col.names=FALSE, quote=FALSE)
}

context("write2csv")

test_that("write2csv works", {

    x <- data.frame(a=2:10,
                    b=rnorm(9),
                    c=letters[1:9],
                    stringsAsFactors=FALSE)

    # no row names
    f <- tempfile()
    write2csv(x, f)
    y <- read.csv(f, stringsAsFactors=FALSE, comment.char="#")
    expect_equal(x, y)

    # row names in output
    write2csv(x, f, row.names="id", overwrite=TRUE)
    y <- read.csv(f, stringsAsFactors=FALSE, comment.char="#")
    expect_equal(cbind(id=as.numeric(rownames(x)), x, stringsAsFactors=FALSE), y)

    # row names in input but not output
    rownames(x) <- paste0("V", sample(9))
    write2csv(x, f, overwrite=TRUE)
    y <- read.csv(f, stringsAsFactors=FALSE, comment.char="#")
    rownames(y) <- rownames(x)
    expect_equal(x, y)

    # row names in input and output
    write2csv(x, f, row.names="id", overwrite=TRUE)
    y <- read.csv(f, stringsAsFactors=FALSE, comment.char="#")
    rownames(y) <- y[,1]
    y <- y[,-1]
    expect_equal(x, y)

    # clean up
    unlink(f)

})

#' Marker map data frame to list
#'
#' Convert a marker map organized as data frame to a list
#'
#' @param map Data frame with marker map
#'
#' @param chr_column Name of the column in `map` that contains the chromosome IDs.
#'
#' @param pos_column Name of the column in `map` that contains the marker positions.
#'
#' @param marker_column Name of the column in `map` that contains
#' the marker names. If NULL, use the row names.
#'
#' @param Xchr Vector of character strings indicating the name or
#' names of the X chromosome. If NULL, assume there's no X
#' chromosome.
#'
#' @seealso [map_list_to_df()]
#'
#' @return A list of vectors of marker positions, one component per chromosome
#'
#' @examples
#' map <- data.frame(chr=c(1,1,1,  2,2,2,   "X","X"),
#'                   pos=c(0,5,10, 0,8,16,  5,20),
#'                   marker=c("D1M1","D1M2","D1M3",    "D2M1","D2M2","D2M3",   "DXM1","DXM2"))
#' map_list <- map_df_to_list(map, pos_column="pos")
#'
#' @export
map_df_to_list <-
    function(map, chr_column="chr", pos_column="cM", marker_column="marker",
             Xchr=c("x", "X"))
{
    if(is.null(marker_column)) {
        marker_column <- "qtl2tmp_marker"
        map[,marker_column] <- rownames(map)
    }
    if(!(marker_column %in% colnames(map)))
        stop('Column "', marker_column, '" not found.')
    if(!(chr_column %in% colnames(map)))
        stop('Column "', chr_column, '" not found.')
    if(!(pos_column %in% colnames(map)))
        stop('Column "', pos_column, '" not found.')

    marker <- map[,marker_column]

    chr <- map[,chr_column]
    uchr <- unique(chr)
    pos <- map[,pos_column]

    result <- split(as.numeric(pos), factor(chr, levels=uchr))
    marker <- split(marker, factor(chr, levels=uchr))
    for(i in seq(along=result))
        names(result[[i]]) <- marker[[i]]

    is_x_chr <- rep(FALSE, length(result))
    names(is_x_chr) <- names(result)
    if(!is.null(Xchr)) {
        Xchr_used <- Xchr %in% names(is_x_chr)
        if(any(Xchr_used)) {
            Xchr <- Xchr[Xchr_used]
            is_x_chr[Xchr] <- TRUE
        }
    }
    attr(result, "is_x_chr") <- is_x_chr

    result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross2_ril_to_genril.R
\name{cross2_ril_to_genril}
\alias{cross2_ril_to_genril}
\title{Convert cross2 object from ril to genril}
\usage{
cross2_ril_to_genril(cross)
}
\arguments{
\item{cross}{Object of class \code{"cross2"}, as produced by
\code{\link[qtl2:read_cross2]{qtl2::read_cross2()}}.}
}
\value{
The input object \code{cross} with cross type changed to
class \code{"genriln"} for some value of \code{n},
and the cross information revised to match.
}
\description{
Convert a cross2 object from cross type \code{"risibn"} to cross type \code{"genriln"}.
}
\examples{
\dontrun{
file <- paste0("https://raw.githubusercontent.com/rqtl/",
               "qtl2data/master/CC/cc.zip")
cc <- read_cross2(file)

cc_genril <- cross2_ril_to_genril(cc)
}

\dontrun{
file <- paste0("https://raw.githubusercontent.com/rqtl/",
               "qtl2data/master/ArabMAGIC/arabmagic_tair9.zip")
arab <- read_cross2(file)

arab_genril <- cross2_ril_to_genril(arab)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probs_qtl2_to_array.R
\name{probs_qtl2_to_array}
\alias{probs_qtl2_to_array}
\title{Convert R/qtl2 genotype probabilities to a 3d array}
\usage{
probs_qtl2_to_array(probs)
}
\arguments{
\item{probs}{A \code{"calc_genoprob"} object (a list of 3d arrays of
genotype probabilities), as calculated by \code{\link[qtl2:calc_genoprob]{qtl2::calc_genoprob()}}.}
}
\value{
A single three-dimensional array, with just the autosomal
genotype probabilities.
}
\description{
Convert R/qtl2 genotype probabilities to a 3d array
}
\details{
We convert just the autosomal genotype probabilities, because they
should all have the same number of genotypes (columns). The main
application of this is for identifying possible sample mix-ups
among batches of genotype probabilities (e.g., using the
\href{https://github.com/kbroman/lineup2}{R/lineup2} package), and for
this the autosomal genotype probabilities should be sufficient.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probs_qtl_to_qtl2.R
\name{probs_qtl_to_qtl2}
\alias{probs_qtl_to_qtl2}
\title{Convert R/qtl genotype probabilities to R/qtl2 format}
\usage{
probs_qtl_to_qtl2(cross)
}
\arguments{
\item{cross}{An R/qtl \code{"cross"} object (see
\code{\link[qtl:read.cross]{qtl::read.cross()}} for details.) Must contain
genotype probabilities as calculated by
\code{\link[qtl:calc.genoprob]{qtl::calc.genoprob()}}.}
}
\value{
A list with two components:
\itemize{
\item \code{"probs"} - the genotype probabilities in the form produced by \code{\link[qtl2:calc_genoprob]{qtl2::calc_genoprob()}}
\item \code{"map"} - Map of marker/pseudomarker positions (a list of vectors of positions)
}
}
\description{
Convert R/qtl genotype probabilities to R/qtl2 format
}
\examples{
library(qtl)
data(hyper)
hyper <- calc.genoprob(hyper, step=1, error.prob=0.002)
result <- probs_qtl_to_qtl2(hyper)
pr <- result$probs
map <- result$map

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_consensus_geno.R
\name{find_consensus_geno}
\alias{find_consensus_geno}
\title{Find the consensus genotype for each row of a genotype matrix}
\usage{
find_consensus_geno(genotypes, na.strings = c("N", "H", "NA", ""), cores = 1)
}
\arguments{
\item{genotypes}{Matrix of genotypes (markers x individuals)}

\item{na.strings}{Genotypes to be considered as missing values.}

\item{cores}{Number of CPU cores to use, for parallel calculations.
(If \code{0}, use \code{\link[parallel:detectCores]{parallel::detectCores()}}.)
Alternatively, this can be links to a set of cluster sockets, as
produced by \code{\link[parallel:makeCluster]{parallel::makeCluster()}}.}
}
\value{
Vector of consensus genotypes, one value per row of \code{genotypes}
}
\description{
For genotype data (markers x individuals) on a set of individuals
from a single inbred line, find the consensus genotype at each
marker.
}
\examples{
g <- rbind(c("NA", "N", "A", "A", "T", "G", NA, "H"),
           c("C",  "C", "G", "G", "A", NA,  NA, NA),
           rep(NA, 8),
           c("C", "C", "G", "G", "G", "C", "G", "G"))
consensus <- find_consensus_geno(g)
}
\seealso{
\code{\link[=find_unique_geno]{find_unique_geno()}}, \code{\link[=encode_geno]{encode_geno()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encode_geno.R
\name{encode_geno}
\alias{encode_geno}
\title{Encode a matrix of genotypes using a set of allele codes}
\usage{
encode_geno(
  geno,
  allele_codes,
  output_codes = c("-", "A", "H", "B"),
  cores = 1
)
}
\arguments{
\item{geno}{Character matrix of genotypes (rows as markers, columns as individuals)}

\item{allele_codes}{Two-column matrix of alleles (rows as markers)}

\item{output_codes}{Vector of length four, with missing, AA, AB, BB codes}

\item{cores}{Number of CPU cores to use, for parallel calculations.
(If \code{0}, use \code{\link[parallel:detectCores]{parallel::detectCores()}}.)
Alternatively, this can be links to a set of cluster sockets, as
produced by \code{\link[parallel:makeCluster]{parallel::makeCluster()}}.}
}
\value{
Matrix of same dimensions as \code{geno}, but with values in \code{output_codes}.
}
\description{
Encode a matrix of genotypes using a set of allele codes.
}
\examples{
geno <- rbind(c("C", "G", "C",  "GG", "CG"),
              c("A", "A", "AT", "TA", "TT"),
              c("T", "G", NA,   "GT", "TT"))
codes <- rbind(c("C", "G"), c("A", "T"), c("T", "G"))
geno_encoded <- encode_geno(geno, codes)

}
\seealso{
\code{\link[=find_consensus_geno]{find_consensus_geno()}}, \code{\link[=find_unique_geno]{find_unique_geno()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qtl2convert-package.R
\docType{package}
\name{qtl2convert-package}
\alias{qtl2convert}
\alias{qtl2convert-package}
\title{qtl2convert: Convert Data among QTL Mapping Packages}
\description{
Functions to convert data structures among the 'qtl2', 'qtl', and 'DOQTL' packages for mapping quantitative trait loci (QTL).
}
\section{Vignettes}{

\itemize{
\item \href{https://kbroman.org/qtl2/assets/vignettes/user_guide.html}{user guide}
\item \href{https://kbroman.org/qtl2/assets/vignettes/input_files.html}{input file formats}
\item \href{https://kbroman.org/qtl2/pages/prep_do_data.html}{preparing DO mouse data for R/qtl2}
\item \href{https://kbroman.org/qtl2/assets/vignettes/rqtl_diff.html}{differences between R/qtl and R/qtl2}
\item \href{https://kbroman.org/qtl2/assets/vignettes/developer_guide.html}{developer guide}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://kbroman.org/qtl2/}
  \item \url{https://github.com/rqtl/qtl2convert}
  \item Report bugs at \url{https://github.com/rqtl/qtl2convert/issues}
}

}
\author{
\strong{Maintainer}: Karl W Broman \email{broman@wisc.edu} (\href{https://orcid.org/0000-0002-4914-6671}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross2_do_to_genail8.R
\name{cross2_do_to_genail8}
\alias{cross2_do_to_genail8}
\alias{cross2_do_to_genail}
\title{Convert cross2 object from do to genail8}
\usage{
cross2_do_to_genail8(cross)

cross2_do_to_genail(cross)
}
\arguments{
\item{cross}{Object of class \code{"cross2"}, as produced by
\code{\link[qtl2:read_cross2]{qtl2::read_cross2()}}.}
}
\value{
The input object \code{cross} with cross type changed to
class \code{"genail8"} and the cross information revised to match.
}
\description{
Convert a cross2 object from cross type \code{"do"} to cross type \code{"genail8"}.
}
\examples{
\dontrun{
file <- paste0("https://raw.githubusercontent.com/rqtl/",
               "qtl2data/master/DOex/DOex.zip")
DOex <- read_cross2(file)

DOex_genail <- cross2_do_to_genail8(DOex)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_list_to_df.R
\name{map_list_to_df}
\alias{map_list_to_df}
\title{Marker map list to data frame}
\usage{
map_list_to_df(
  map_list,
  chr_column = "chr",
  pos_column = "pos",
  marker_column = "marker"
)
}
\arguments{
\item{map_list}{List of vectors containing marker positions}

\item{chr_column}{Name of the chromosome column in the output}

\item{pos_column}{Name of the position column in the output}

\item{marker_column}{Name of the marker column in the output.
If NULL, just put them as row names.}
}
\value{
A data frame with the marker positions.
}
\description{
Convert a marker map organized as a list to a data frame
}
\examples{
library(qtl2)
iron <- read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
iron_map <- map_list_to_df(iron$gmap)

}
\seealso{
\code{\link[=map_df_to_list]{map_df_to_list()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probs_qtl2_to_doqtl.R
\name{probs_qtl2_to_doqtl}
\alias{probs_qtl2_to_doqtl}
\title{Convert R/qtl2 genotype probabilities to DOQTL format}
\usage{
probs_qtl2_to_doqtl(probs)
}
\arguments{
\item{probs}{A \code{"calc_genoprob"} object (a list of 3d arrays of
genotype probabilities), as calculated by \code{\link[qtl2:calc_genoprob]{qtl2::calc_genoprob()}}.}
}
\value{
A single three-dimensional array, for use with
\href{https://github.com/dmgatti/DOQTL}{DOQTL}.
}
\description{
Convert R/qtl2 genotype probabilities to DOQTL format
}
\details{
If the arrays in \code{probs} all have 8 columns, they're assumed to be
allele dosages and we paste them all together into one big array.

Otherwise, it should be that the autosomes all have 36 columns the
X chromosome has 44. In this case, the male hemizygotes on the X
are placed where the female homozygotes are, and then we reorder
the genotypes into alphabetical order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write2csv.R
\name{write2csv}
\alias{write2csv}
\title{Write a data frame to a CSV file}
\usage{
write2csv(
  df,
  filename,
  comment = "",
  sep = ",",
  comment.char = "#",
  row.names = NULL,
  overwrite = FALSE
)
}
\arguments{
\item{df}{A data frame (or matrix)}

\item{filename}{File name to write}

\item{comment}{Comment to place on the first line}

\item{sep}{Field separator}

\item{comment.char}{Character to use to initiate the comment lines}

\item{row.names}{If NA or NULL (the default), row names are not
included in the output file. Otherwise, the row names are
included as the first column of the output, and this is taken
to be the name for that column.}

\item{overwrite}{If TRUE, overwrite file if it exists. If FALSE
(the default) and the file exists, stop with an error.}
}
\value{
None.
}
\description{
Write a data frame to a CSV file in a special form, with info about the number of rows and columns.
}
\details{
If the file already exists, the function will refuse to write over it.

The file will include comments at the top, using \verb{#} as a
comment character, including the number of rows (not including the
header) and the number of columns.

By default, row names are not included. But with the option
\code{row.names} provided as a character string, they are added as an
initial column, with the value of this argument defining the name
for that column. If a column with that name already exists, the
function halts with an error.
}
\examples{
nr <- 10
nc <- 5
x <- data.frame(id=paste0("ind", 1:nr),
                matrix(rnorm(nr*nc), ncol=nc))
colnames(x)[1:nc + 1] <- paste0("col", 1:nc)

testfile <- file.path(tempdir(), "tmpfile.csv")
write2csv(x, testfile, "A file created by write2csv")

# Remove the file, to clean up temporary directory
unlink(testfile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_unique_geno.R
\name{find_unique_geno}
\alias{find_unique_geno}
\title{Find the unique genotypes for each row of a genotype matrix}
\usage{
find_unique_geno(genotypes, na.strings = c("N", "H", "NA", ""), cores = 1)
}
\arguments{
\item{genotypes}{Matrix of genotypes (markers x individuals)}

\item{na.strings}{Genotypes to be considered as missing values.}

\item{cores}{Number of CPU cores to use, for parallel calculations.
(If \code{0}, use \code{\link[parallel:detectCores]{parallel::detectCores()}}.)
Alternatively, this can be links to a set of cluster sockets, as
produced by \code{\link[parallel:makeCluster]{parallel::makeCluster()}}.}
}
\value{
Matrix with two columns. Each row corresponds to a marker,
and has the two unique genotypes, or \code{NA}s (if >2 or <2 unique
genotypes).
}
\description{
For genotype data (markers x individuals) on a set of individuals,
find the unique genotypes for each marker, provided that there are exactly two.
(If more than two or fewer than two, return NAs.)
}
\examples{
g <- rbind(c("NA", "A",  "A",  "A", "T"),
           c("NA", "NA", "NA", "A", "A"),
           c("A",  "A",  "T",  "G", "G"),
           c("C", "C",  "G",  "G", "NA"))
ug <- find_unique_geno(g)
}
\seealso{
\code{\link[=count_unique_geno]{count_unique_geno()}}, \code{\link[=encode_geno]{encode_geno()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbind_smother.R
\name{cbind_smother}
\alias{cbind_smother}
\title{Combine matrices by columns, replacing matching ones and adding unique ones}
\usage{
cbind_smother(mat1, mat2)
}
\arguments{
\item{mat1}{A matrix}

\item{mat2}{Another matrix, with the same number of rows as \code{mat}.}
}
\value{
The two matrices combined by columns, but columns in the
first matrix that also appear in the second matrix are deleted
and replaced by those in the second matrix. Uses the row names
to align the rows in the two matrices, and to expand them as needed.
}
\description{
This is like \code{\link[base:cbind]{base::cbind()}} but if a column in the second matrix
has the same name as a column in the first matrix, the column in
the first matrix is deleted and that in the second matrix is used
in its place.
}
\examples{
df1 <- data.frame(x=c(1,2,3,NA,4), y=c(5,8,9,10,11), row.names=c("A", "B", "C", "D", "E"))
df2 <- data.frame(z=c(7,8,0,9,10), y=c(6,NA,NA,9,10), row.names=c("A", "B", "F", "C", "D"))
df1n2 <- cbind_smother(df1, df2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scan_qtl_to_qtl2.R
\name{scan_qtl_to_qtl2}
\alias{scan_qtl_to_qtl2}
\title{Convert R/qtl scanone results to R/qtl2 scan1 format}
\usage{
scan_qtl_to_qtl2(scanone_output)
}
\arguments{
\item{scanone_output}{Data frame as output by the R/qtl1 function \code{\link[qtl:scanone]{qtl::scanone()}}.}
}
\value{
List with two objects: the LOD scores in \code{\link[qtl2:scan1]{qtl2::scan1()}}
format, and the map (as a list of marker/pseudomarker
positions).
}
\description{
Convert the results of R/qtl1 \code{\link[qtl:scanone]{qtl::scanone()}} to the form used by the R/qtl2 \code{\link[qtl2:scan1]{qtl2::scan1()}}.
}
\examples{
library(qtl)
data(hyper)
hyper <- calc.genoprob(hyper, step=1, error.prob=0.002)
out <- scanone(hyper)
out2 <- scan_qtl_to_qtl2(out)

}
\seealso{
\code{\link[=scan_qtl_to_qtl2]{scan_qtl_to_qtl2()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scan_qtl2_to_qtl.R
\name{scan_qtl2_to_qtl}
\alias{scan_qtl2_to_qtl}
\title{Convert scan1 results to the scanone format}
\usage{
scan_qtl2_to_qtl(scan1_output, map)
}
\arguments{
\item{scan1_output}{Matrix of LOD scores, as calculated by
\code{\link[qtl2:scan1]{qtl2::scan1()}}.}

\item{map}{Map of markers/pseudomarkers (as a list of vectors).}
}
\value{
A data frame with class \code{"scanone"}, containing
chromosome and position columns followed by the LOD scores in
\code{scan1_output}.
}
\description{
Convert the results of \code{\link[qtl2:scan1]{qtl2::scan1()}}
to the form used by the R/qtl function
\code{\link[qtl:scanone]{qtl::scanone()}}.
}
\examples{
library(qtl2)
iron <- read_cross2(system.file("extdata", "iron.zip", package="qtl2"))
map <- insert_pseudomarkers(iron$gmap, step=1)
probs <- calc_genoprob(iron, map, error_prob=0.002)
pheno <- iron$pheno
covar <- match(iron$covar$sex, c("f", "m")) # make numeric
names(covar) <- rownames(iron$covar)
Xcovar <- get_x_covar(iron)
out <- scan1(probs, pheno, addcovar=covar, Xcovar=Xcovar)

out_rev <- scan_qtl2_to_qtl(out, map)

}
\seealso{
\code{\link[=scan_qtl2_to_qtl]{scan_qtl2_to_qtl()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map_df_to_list.R
\name{map_df_to_list}
\alias{map_df_to_list}
\title{Marker map data frame to list}
\usage{
map_df_to_list(
  map,
  chr_column = "chr",
  pos_column = "cM",
  marker_column = "marker",
  Xchr = c("x", "X")
)
}
\arguments{
\item{map}{Data frame with marker map}

\item{chr_column}{Name of the column in \code{map} that contains the chromosome IDs.}

\item{pos_column}{Name of the column in \code{map} that contains the marker positions.}

\item{marker_column}{Name of the column in \code{map} that contains
the marker names. If NULL, use the row names.}

\item{Xchr}{Vector of character strings indicating the name or
names of the X chromosome. If NULL, assume there's no X
chromosome.}
}
\value{
A list of vectors of marker positions, one component per chromosome
}
\description{
Convert a marker map organized as data frame to a list
}
\examples{
map <- data.frame(chr=c(1,1,1,  2,2,2,   "X","X"),
                  pos=c(0,5,10, 0,8,16,  5,20),
                  marker=c("D1M1","D1M2","D1M3",    "D2M1","D2M2","D2M3",   "DXM1","DXM2"))
map_list <- map_df_to_list(map, pos_column="pos")

}
\seealso{
\code{\link[=map_list_to_df]{map_list_to_df()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probs_doqtl_to_qtl2.R
\name{probs_doqtl_to_qtl2}
\alias{probs_doqtl_to_qtl2}
\title{Convert DOQTL genotype probabilities to R/qtl2 format}
\usage{
probs_doqtl_to_qtl2(
  probs,
  map,
  is_female = NULL,
  chr_column = "chr",
  pos_column = "cM",
  marker_column = "marker"
)
}
\arguments{
\item{probs}{3d array of genotype probabilities as calculated from DOQTL
(individuals x genotypes x positions)}

\item{map}{Data frame with marker map}

\item{is_female}{Optional logical vector indicating which
individuals are female. Names should contain the individual
identifiers, matching the row names in \code{probs}.}

\item{chr_column}{Name of the column in \code{map} that contains the chromosome IDs.}

\item{pos_column}{Name of the column in \code{map} that contains the marker positions.}

\item{marker_column}{Name of the column in \code{map} that contains
the marker names. If NULL, use the row names.}
}
\value{
An object of the form produced by \code{\link[qtl2:calc_genoprob]{qtl2::calc_genoprob()}}.
}
\description{
Convert DOQTL genotype probabilities to R/qtl2 format
}
\details{
We assume that the X chromosome is labeled \code{"X"} (must be
upper-case) and that any other chromosomes are autosomes.
We assume that the genotypes are labeled using the 8 letters A-H.

If the probabilities are for the full 36 states and the X
chromosome is included but \code{is_female} is not provided, we'll guess
which individuals are females based on their genotype
probabilities. (If the average, across loci, of the sum of the
heterozygote probabilities is small, we'll assume it's a female.)
}
